import { Inject, Injectable, UnauthorizedException } from '@nestjs/common';
import { PassportStrategy } from '@nestjs/passport';
import { FastifyRequest } from 'fastify';
import { ExtractJwt, Strategy } from 'passport-jwt';
import { authConfig, AuthConfig } from '../../auth.config';
import { RequestWithUser } from '../../constants/request-with-user';
import { TokenPayload } from '../../constants/token-payload.interface';
import { UsersAuthService } from '../../users/users-auth.service';

@Injectable()
export class JwtRefreshTokenAuthStrategy extends PassportStrategy(Strategy, 'jwt-refresh-token') {
  constructor(@Inject(authConfig.KEY) authConfig: AuthConfig, private readonly usersAuthService: UsersAuthService) {
    super({
      jwtFromRequest: ExtractJwt.fromExtractors([
        (request: FastifyRequest) => {
          return request?.headers?.Bearer as string;
        },
      ]),
      secretOrKey: authConfig.refreshToken.secret,
      passReqToCallback: true,
    });
  }

  async validate(request: RequestWithUser, payload: TokenPayload) {
    const refreshToken = request.headers?.Refresh as string;
    const account = this.usersAuthService.getUserByIdAndRefreshTokenPair(payload.userId, refreshToken);
    if (!account) throw new UnauthorizedException('Refresh Token Guard');
    return account;
  }
}
